@@ -4,7 +4,7 @@ use 5.006;
 use strict;
 use warnings;
 
-use List::Util      qw/min/;
+use List::Util      qw/any min/;
 use List::MoreUtils qw/zip first_index/;
 use Scalar::Util    qw/looks_like_number/;
 
@@ -14,11 +14,11 @@ Games::Cards::ShuffleTrack - Track cards through shuffles and cuts
 
 =head1 VERSION
 
-Version 0.01_6
+Version 0.03_1
 
 =cut
 
-our $VERSION = '0.01_6';
+our $VERSION = '0.03_1';
 
 my $cut_limits = {
 	normal  => [0.19, 0.82], # on a 52 cards deck, cut between 10 and 43 cards
@@ -49,7 +49,7 @@ my $shortcuts = {
 
 =head1 SYNOPSIS
 
-This module allows you to simulate true and false shuffles and cuts.
+This module allows you to simulate shuffles and cuts.
 
 	use Games::Cards::ShuffleTrack;
 
@@ -59,7 +59,7 @@ This module allows you to simulate true and false shuffles and cuts.
 	$deck->riffle_shuffle();
 	$deck->cut( 'short' );
 	$deck->riffle_shuffle();
-	print $deck->get_deck();
+	print "@{$deck->get_deck()}";
 
 Or perhaps with more precision:
 
@@ -69,7 +69,7 @@ Or perhaps with more precision:
 	$deck->cut( 26 );
 	print $deck->get_deck();
 
-See the rest of the documentation for more advanced features.
+See the rest of the documentation for more advanced features. See the examples folder for more detailed usage.
 
 
 =head1 INSTALLATION
@@ -153,7 +153,7 @@ Reset the deck to its original status. The original is whatever you selected whe
 
 	$deck->restart;
 
-Do bear in mind that by doing so you are replinishing the deck of whatever cards you took out of it.
+Do bear in mind that by doing so you are replenishing the deck of whatever cards you took out of it.
 
 =cut
 
@@ -167,9 +167,7 @@ sub restart {
 
 Returns the size of the deck.
 
-    for ( 1 .. $deck->deck_size ) {
-        ...
-    }
+    my $size = $deck->deck_size;
 
 =cut
 
@@ -187,7 +185,9 @@ Returns the deck (a reference to a list of strings).
 
 =cut
 
-sub get_deck { # TODO: use wantarray to allow for an array to be returned
+# TODO: should we return a copy of the list instead of the list itself?
+# TODO: use wantarray to allow for an array to be returned?
+sub get_deck {
 	my $self = shift;
 	return $self->{'deck'};
 }
@@ -321,16 +321,34 @@ Running cards basically reverses their order.
 
 If no number is given then no cards are run.
 
+If we're doing multiple runs we can set everything at the same time:
+
+	$deck->run( 4, 6, 2 );
+
 =cut
 
+# TODO: review this code
 sub run {
-	my $self            = shift;
-	my $number_of_cards = shift || return $self;
-	my $where_to_drop   = shift || 'drop-bottom';
+	my $self = shift;
 
+	my @number_of_cards;
+	my $where_to_drop;
+
+	while ( my $param = shift ) {
+		if ( looks_like_number( $param ) ) {
+			push @number_of_cards, $param;
+		}
+		else {
+			$where_to_drop = $param;
+		}
+	}
+
+	@number_of_cards || return $self;
+	$where_to_drop   ||= 'drop-bottom';
+
+	my $number_of_cards = shift @number_of_cards;
 	$number_of_cards > 0 or return $self;
 
-
 	# take cards from top and reverse their order
 	my @deck = @{$self->get_deck};
 	my @run  = reverse splice @deck, 0, $number_of_cards;
@@ -342,7 +360,7 @@ sub run {
 		$self->_set_deck( @deck, @run );
 	}
 
-	return $self;
+	return $self->run( @number_of_cards, $where_to_drop );
 }
 
 
@@ -494,6 +512,7 @@ For more information on how to cut to a specific card please refer to the L<SEE
 
 =cut
 
+# TODO: delimit randomness of the cut between two numbers: $deck->cut( 1, 25 );
 sub cut {
 	my $self     = shift;
 	my $position = shift;
@@ -527,6 +546,109 @@ sub cut_above {
 	return $self->cut( $self->find( $card ) - 1 );
 }
 
+=head3 cut_to
+
+Cuts a portion of the deck to another position
+
+	$deck->cut_to( $pile );
+
+You can specify exactly how many cards to cut or delimit the randomness of the cut:
+
+	# cut exactly 15 cards to $pile
+	$deck->cut_to( $pile, 15 );
+
+	# cut between 10 and 26 cards to $pile
+	$deck->cut_to( $pile, 10, 26 );
+
+If the position doesn't exist yet you can also automatically create it:
+
+	my $pile = $deck->cut_to();
+
+This method returns the new pile.
+
+=cut
+
+sub cut_to {
+	my $self = shift;
+
+	# create the new pile if required
+	my $new_pile;
+	if ( defined($_[0]) and ref( $_[0] ) eq 'Games::Cards::ShuffleTrack' ) {
+		$new_pile = shift;
+	}
+	else {
+		$new_pile = Games::Cards::ShuffleTrack->new( 'empty' );
+	}
+
+	# TODO: could this be done with _cut_depth? (perhaps changing it a bit)
+	# set the position
+	my $lower_limit = shift;
+	my $upper_limit = shift;
+
+	my $position;
+
+	if ( not defined $lower_limit ) {
+		$position = _rand( 1, $self->deck_size );
+	}
+	elsif ( not defined $upper_limit ) {
+		$position = $lower_limit;
+	}
+	else {
+		$position = _rand( $lower_limit, $upper_limit );
+	}
+
+	# cut the deck
+	$new_pile->place_on_top( splice @{$self->get_deck}, 0, $position );
+
+	return $new_pile;
+}
+
+
+=head3 place_on_top
+
+Places a pile of cards on top of the deck.
+
+	$deck->place_on_top( qw/AS KS QS JS 10S/ );
+
+=cut
+
+sub place_on_top {
+	my $self = shift;
+	my @pile = @_;
+
+	$self->_set_deck( @pile, @{$self->get_deck} );
+
+	return $self;
+}
+
+=head3 complete_cut, move_to
+
+Complete the cut by moving all cards from one deck onto another:
+
+	$deck->complete_cut( $new_pile );
+	
+	# or
+
+	$deck->move_to( $table );
+
+=cut
+
+sub complete_cut {
+	my $self = shift;
+	my $destination = shift;
+
+	$self->cut_to( $destination, $self->deck_size );
+
+	return $self;
+}
+
+sub move_to {
+	my $self = shift;
+
+	return $self->complete_cut( @_ );
+}
+
+
 =head3 running_cuts
 
 Cut packets:
@@ -559,13 +681,15 @@ There are a few different methods to track down cards.
 
 Get the position of specific cards:
 
-	$deck->find( 'AS' ); # find the position of the Ace of Spades
+	my $position = $deck->find( 'AS' ); # find the position of the Ace of Spades
 
-	$deck->find( 'AS', 'KH' ); # find the position of two cards
+	my @positions = $deck->find( 'AS', 'KH' ); # find the position of two cards
+
+If a card is not present on the deck the position returned will be a 0.
 
 This method can also return the card at a specific position:
 
-	$deck->find( 3 );
+	my $card = $deck->find( 3 );
 
 You can also request a card in a negative position (i.e., from the bottom of the deck). To get the second to last card in the deck:
 
@@ -602,10 +726,12 @@ sub _find_card_by_position {
 
 	if ($card) {
 		if ($card > 0) { $card--; }
-		return $self->get_deck->[ $card ];
+		return $card > $self->deck_size - 1 ?
+		        q{} :
+		        $self->get_deck->[ $card ];
 	}
 	else {
-		return undef;
+		return q{};
 	}
 }
 
@@ -615,7 +741,7 @@ sub _find_card_by_name {
 
 	my $position = 1 + first_index { $_ eq $card } @{$self->get_deck};
 
-	return $position ? $position : undef;
+	return $position ? $position : 0;
 }
 
 
@@ -637,7 +763,7 @@ sub find_card_before {
 	my $position = $self->find( $card );
 
 	if ($position == 1) {
-		return undef;
+		return 0;
 	}
 	else {
 		return $self->find( $position - 1 );
@@ -660,6 +786,12 @@ sub find_card_after {
 	my $self = shift;
 	my $card = shift;
 
+    my $position = 1 + $self->find( $card );
+    
+    if ( $position > $self->deck_size ) {
+        return 0;
+    }
+
 	return $self->find( $self->find( $card ) + 1 );
 }
 
@@ -705,6 +837,40 @@ sub put {
 }
 
 
+=head3 insert
+
+Inserts a card in a specified position in the deck. If the position isn't specified than the card is inserted somewhere at random.
+
+	# insert a Joker at position 20
+	$deck->insert( 'Joker', 20 );
+
+	# replace a card somewhere in the deck at random
+	$deck->insert( $card );
+
+If the position doesn't exist the card will be replaced at the bottom of the deck.
+
+=cut
+
+# TODO: inserting in negative position?
+# TODO: what if the user inserts at position 0?
+sub insert {
+	my $self     = shift;
+	my $card     = shift;
+	my $position = shift;
+
+	if ( not defined $position ) {
+		$position = _rand( 1, $self->deck_size );
+	}
+	elsif ( $position > $self->deck_size ) {
+		$position = $self->deck_size + 1;
+	}
+
+	splice @{$self->get_deck}, $position - 1, 0, $card;
+
+	return $self;
+}
+
+
 =head3 deal
 
 Deals a card, removing it from the deck.
@@ -724,16 +890,43 @@ Just as in regular gambling, you can deal cards from other positions:
 
 For more information on false dealing see the L<SEE ALSO> section.
 
+If you're dealing cards to a pile you can actually state where you're dealing:
+
+	$deck->deal( $pile );
+
+You can still do a false deal to a pile:
+
+	$deck->deal( 'second', $pile );
+
+	# or
+
+	$deck->deal( $pile, 'second' );
+
 =cut
 
 sub deal {
 	my $self = shift;
-	my $deal = shift || 'top';
 
-	# TODO: what if $deal is unknown?
+	my $deal        = 'top';
+	my $destination = undef;
+	while (my $param = shift) {
+		if ( ref( $param ) eq 'Games::Cards::ShuffleTrack' ) {
+			$destination = $param;
+		}
+		elsif ( exists $shortcuts->{$param} ) {
+			$deal = $param;
+		}
+	}
+
 	my $position = $shortcuts->{$deal};
 
-	return $self->remove( $position );
+	my $card = $self->remove( $position );
+	if ( defined $destination ) {
+		return $destination->put( $card );
+	}
+	else {
+		return $card;
+	}
 }
 
 =head3 remove
@@ -745,8 +938,9 @@ Removes a card from the deck.
 
 =cut
 
+# TODO: allow removal of several cards (do note that positions change as cards are removed)
 sub remove {
-	my $self = shift;
+	my $self     = shift;
 	my $position = shift;
 
 	my @deck = @{$self->get_deck};
@@ -841,7 +1035,7 @@ sub _cut_depth {
 		$position = 'normal';
 	}
 
-	if ($position =~ /^short|center|deep|normal$/) {
+	if ( any { $_ eq $position } keys %$cut_limits ) {
 		my ($lower, $upper) = @{$cut_limits->{ $position }};
 		$position = _rand( $deck_size * $lower, $deck_size * $upper );
 	}
@@ -891,13 +1085,13 @@ L<http://search.cpan.org/dist/Games-Cards-ShuffleTrack/>
 
 =head1 SEE ALSO
 
-Recommended reading:
+The following is a very, very, very, very extremely small list of recommended books:
 
 =over 4
 
-=item * The Expert at the Card Table: The Classic Treatise on Card Manipulation, by S. W. Erdnase
+=item * The Expert at the Card Table, by S. W. Erdnase
 
-=item * The Annotated Erdnase, by Darwin Ortiz
+=item * Card College, by Roberto Giobbi
 
 =back
 
